local bs = AceLibrary("Babble-Spell-2.2")
local AceAddon = AceLibrary("AceAddon-2.0")

local L = AceLibrary("AceLocale-2.0"):new("Cartographer_Mining")
L:RegisterTranslations("enUS", function() return {
    ["^You perform %s on (.+)%%.$"] = true,

    ["Vein"] = true,
    ["Deposit"] = true,

    ["Copper"] = true,
    ["Tin"] = true,
    ["Iron"] = true,
    ["Silver"] = true,
    ["Gold"] = true,
    ["Mithril"] = true,
    ["Truesilver"] = true,
    ["Thorium"] = true,
    ["Small Thorium"] = true,
    ["Rich Thorium"] = true,
    ["Dark Iron"] = true,
} end)

local mod = Cartographer:NewModule(bs["Mining"], "AceConsole-2.0", "AceEvent-2.0")

mod.icon = {
    ["Copper"] = {
        text = L["Copper"],
        path = "Interface\\Icons\\INV_Ore_Copper_01",
        width = 12,
        height = 12
    },
    ["Tin"] = {
        text = L["Tin"],
        path = "Interface\\Icons\\INV_Ore_Tin_01",
        width = 12,
        height = 12
    },
    ["Silver"] = {
        text = L["Silver"],
        path = "Interface\\Icons\\INV_Stone_16",
        width = 12,
        height = 12
    },
    ["Iron"] = {
        text = L["Iron"],
        path = "Interface\\Icons\\INV_Ore_Iron_01",
        width = 12,
        height = 12
    },
    ["Gold"] = {
        text = L["Gold"],
        path = "Interface\\Icons\\INV_Ore_Copper_01",
        width = 12,
        height = 12
    },
    ["Mithril"] = {
        text = L["Mithril"],
        path = "Interface\\Icons\\INV_Ore_Mithril_02",
        width = 12,
        height = 12
    },
    ["Truesilver"] = {
        text = L["Truesilver"],
        path = "Interface\\Icons\\INV_Ore_TrueSilver_01",
        width = 12,
        height = 12
    },
    ["Small Thorium"] = {
        text = L["Small Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Thorium"] = {
        text = L["Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Rich Thorium"] = {
        text = L["Rich Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Dark Iron"] = {
        text = L["Dark Iron"],
        path = "Interface\\Icons\\INV_Ore_Mithril_01",
        width = 12,
        height = 12
    },
}

function mod:OnInitialize()
    local aceopts = AceAddon:GetAceOptionsDataTable(self)
    aceopts.toggle = {
        name  = AceLibrary('AceLocale-2.2'):new('Cartographer')['Toggle'],
        desc  = AceLibrary('AceLocale-2.2'):new('Cartographer')['Toggle the module on and off.'],
        type  = 'toggle',
        order = -1,
        get   = function() return Cartographer:IsModuleActive(self) end,
        set   = function() Cartographer:ToggleModuleActive(self) end,
    }

    Cartographer.options.args[bs["Mining"]] = {
        name = bs["Mining"],
        desc = self.notes,
        type = 'group',
        args = aceopts,
        handler = self,
    }
    Cartographer:GetModule('PinPoint').addons[bs["Mining"]] = self

    if not Cartographer_MiningDB then
        Cartographer_MiningDB = {}
    end

    if Cartographer_Notes then
        for k,v in pairs(self.icon) do
            Cartographer_Notes:RegisterIcon(k, v)
        end
    end
end

function mod:OnEnable()
    if Cartographer_Notes then
        Cartographer_Notes:RegisterNotesDatabase('Mining', Cartographer_MiningDB)
    else
        Cartographer:ToggleModuleActive(self, false)
    end

    self:RegisterEvent("UNIT_SPELLCAST_SENT")
    self:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF")
    self:RegisterEvent("UI_ERROR_MESSAGE")
end

function mod:OnDisable()
    self:UnregisterAllEvents()

    if Cartographer_Notes then
        Cartographer_Notes:UnregisterNotesDatabase('Mining')
    end
end

function mod:SetNote(what)
    local x, y = GetPlayerMapPosition("player")
    if x == 0 and y == 0 then return end
    local zone = GetRealZoneText()
    Cartographer_Notes:SetNote(zone, x, y, what, 0, 0.8, 0, "", 1, 1, 1, "", 1, 1, 1, "Mining", L:GetReverseTranslation(what))
end

function mod:RegisterOreFromTooltip()
    local what = GameTooltipTextLeft1:GetText()
    if what and strlen(what) > 0 then
        if string.find(what, L["Dark Iron"]) then
            self:SetNote(L["Dark Iron"])
        elseif string.find(what, L["Rich Thorium"]) then
            self:SetNote(L["Dark Thorium"])
        elseif string.find(what, L["Small Thorium"] ) then
            self:SetNote(L["Small Thorium"])
        else
            what = string.gsub(what, " %(%d+%)", "")
            local _, _, ore, veindep = string.find(what, "([^ ]+) ([^ ]+)$")
            if ore and veindep and veindep == L["Vein"] or veindep == L["Deposit"] then
                self:SetNote(ore)
            end
        end
    end
end

function mod:CHAT_MSG_SPELL_SELF_BUFF(msg)
    local _,_,what = string.find(msg, string.format(L["^You perform %s on (.+)%%.$"], bs["Mining"]))
    if what then
        self:RegisterOreFromTooltip()
    end
end

function mod:UI_ERROR_MESSAGE(msg)
    if string.find(msg, UNIT_SKINNABLE_ROCK) then
        local what = GameTooltipTextLeft1:GetText()
        if what and strlen(what) > 0 then
            self:RegisterOreFromTooltip()
        end
    end
end

function mod:UNIT_SPELLCAST_SENT(unit, spell, rank, target)
    if spell == bs["Mining"] then
        local what = GameTooltipTextLeft1:GetText()
        if what and strlen(what) > 0 then
            self:RegisterOreFromTooltip()
        end
    end
end
